import matplotlib.colors as colors
import numpy as np
import matplotlib.pyplot as plt
import copy
from matplotlib.colors import BoundaryNorm
import matplotlib.patches as patches




def creat_cmap_from_rgb(rgb_list_list,extend = None):
    if isinstance(rgb_list_list[0],list):
        colors_list = np.array(rgb_list_list)/256
    else:
        colors_list =rgb_list_list
    if extend == 'min':
        cmap = colors.ListedColormap(colors_list[1:])
        cmap.set_under(colors_list[0])
    elif extend == 'max':
        cmap = colors.ListedColormap(colors_list[:-1])
        cmap.set_over(colors_list[-1])
    elif extend == 'both':
        cmap = colors.ListedColormap(colors_list[1:-1])
        cmap.set_under(colors_list[0])
        cmap.set_over(colors_list[-1])
    else:
        cmap = colors.ListedColormap(colors_list)

    return cmap


def score_skill_seaborn(score_array_list,title_list,discription_list,member_list,dtime_list,save_path = None,show = False):

    rgb_list_list = [
        "#00529F", "#0072BB", "#0694CB", "#56B1DA", "#92CCE4", "#C1DCF1", "#DBECF8",
        "#F1F1F1",
        "#FEDECF", "#FFB79B", "#FE8968", "#FF5C3B", "#FF1513", "#DC0002", "#B20000"
    ]
    clevs = [-50, -25, -15, -10, -5, -2, -1, 1, 2, 5, 10, 15, 25, 50]
    cmap = creat_cmap_from_rgb(rgb_list_list)

    xlabels = np.array(dtime_list)/24
    ylabels = member_list[::-1]

    nscore = len(score_array_list)
    skill_list = []
    score_re_list = []  #
    for k in range(nscore):
        score_re = score_array_list[k][::-1 ,:]
        score_re_list.append(score_re)
        skill = 100 * (score_re - score_re[-1, :]) / score_re[-1, :]
        skill_list.append(skill)
    # cmap,clevs = meb.def_cmap_clevs(meb.cmaps.me,clevs=[-50,-25,-15,-10,-5,-2,-1,1,2,5,10,15,25,50])
    fig = plt.figure(figsize=(12.5, 2.5), dpi=600)
    plt.subplots_adjust(wspace=0.05)
    annot_list = [0, 1, 1, 1, 1]
    for k in range(nscore):
        ax_one = plt.subplot(1, nscore, k + 1)
        data_k = copy.deepcopy(skill_list[k])
        nx = data_k.shape[1]
        ny = data_k.shape[0]
        x = np.arange(nx + 1) - 0.5
        y = np.arange(ny + 1) - 0.5
        norm = BoundaryNorm(clevs, ncolors=cmap.N + 1)
        im = ax_one.pcolormesh(x, y, data_k, cmap=cmap, norm=norm, linewidths=2, edgecolor="w")
        ax_one.spines['bottom'].set_linewidth(0)
        ax_one.spines['top'].set_linewidth(0)
        ax_one.spines['right'].set_linewidth(0)
        ax_one.spines['left'].set_linewidth(0)
        fmt_tag = "%." + str(annot_list[k]) + "f"
        data = copy.deepcopy(score_re_list[k])

        for j in range(ny):
            for i in range(nx):
                data_ijk = data_k[j, i]
                if not np.isnan(data_ijk):
                    ax_one.text(i, j, fmt_tag % data[j, i], ha="center", va="center", fontsize=10, c="k")
                rect = patches.Rectangle((i - 0.45, j - 0.45), 0.9, 0.9, linewidth=0.1, edgecolor='gray',
                                         facecolor='none')
                ax_one.add_patch(rect)
            rect = patches.Rectangle((-0.45, j - 0.45), 4.9, 0.9, linewidth=0.3, edgecolor='gray', facecolor='none')
            ax_one.add_patch(rect)
        # ax_one.set_xticks(np.arange(nx))
        plt.xticks(np.arange(nx), xlabels, fontsize=11, family='Times New Roman')
        plt.xlabel("Lead time[days]", fontsize=12)
        if k == 0:
            plt.yticks(np.arange(5).tolist(), ylabels, fontsize=12)
        else:
            plt.yticks([])
        plt.title(discription_list[k], fontsize=8)
        plt.text(2, 5.3, title_list[k], fontsize=14, ha="center", va="center")

    location = [0.2, -0.15, 0.6, 0.05]
    colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
    plt.colorbar(im, cax=colorbar_position, orientation="horizontal",
                 ticks=clevs, label="Better <- % difference in RMSE vs ECMWF -> Worse", extend="both")
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches="tight")
    if show:
        plt.show()
    plt.close()

def rmse_skill_seaborn(rmse_z ,rmse_t ,rmse_q ,rmse_ws ,rmse_t2m ,member_list,dtime_list,save_path= None,show = False):
    score_array_list = [rmse_z, rmse_t, rmse_t2m, rmse_q, rmse_ws]
    title_list = ["Geopotential" ,"Temperature" ,"Temperature" ,"Humidity" ,"Wind Speed"]
    discription_list = ["500hPa geopotential RMSE[m$^{2}$/s$^{2}$]" ,"850hPa temperature RMSE[K]",
                     "2m temperature RMSE[K]",
                     "700hPa specific humidity RMSE[g/kg]" ,"850hPa wind vector RMSE[m/s]"]
    score_skill_seaborn(score_array_list, title_list, discription_list,member_list,dtime_list, save_path=save_path, show=show)


def seaborn_score_bak(rmse_z ,rmse_t ,rmse_q ,rmse_ws ,rmse_t2m ,g_dict,save_path= None,show = False):

    rmse_z = rmse_z[::-1 ,:]
    rmse_t = rmse_t[::-1 ,:]
    rmse_q = rmse_q[::-1 ,:]
    rmse_ws = rmse_ws[::-1 ,:]
    rmse_t2m = rmse_t2m[::-1 ,:]

    rgb_list_list =[
        "#00529F", "#0072BB",  "#0694CB" ,"#56B1DA" ,"#92CCE4" ,"#C1DCF1" ,"#DBECF8",
        "#F1F1F1",
        "#FEDECF" ,"#FFB79B" ,"#FE8968" ,"#FF5C3B" ,"#FF1513" ,"#DC0002" ,"#B20000"
    ]
    clevs =[-50 ,-25 ,-15 ,-10 ,-5 ,-2 ,-1 ,1 ,2 ,5 ,10 ,15 ,25 ,50]
    cmap = creat_cmap_from_rgb(rgb_list_list)


    xlabels = [1 ,3 ,5 ,7 ,10]
    ylabels = g_dict["member"][::-1]
    titles = ["Geopotential" ,"Temperature" ,"Temperature" ,"Humidity" ,"Wind Speed"]
    titles_little = ["500hPa geopotential RMSE[m$^{2}$/s$^{2}$]" ,"850hPa temperature RMSE[K]",
                     "2m temperature RMSE[K]",
                     "700hPa specific humidity RMSE[g/kg]" ,"850hPa wind vector RMSE[m/s]"]

    skill_z = 100 *(rmse_z - rmse_z[-1 ,:] ) /rmse_z[-1 ,:]
    skill_t = 100 *(rmse_t - rmse_t[-1 ,:] ) /rmse_t[-1 ,:]
    skill_t2m = 100 *(rmse_t2m - rmse_t2m[-1 ,:] ) /rmse_t2m[-1 ,:]
    skill_q = 100 *(rmse_q - rmse_q[-1 ,:] ) /rmse_q[-1 ,:]
    skill_ws = 100 *(rmse_ws - rmse_ws[-1 ,:] ) /rmse_ws[-1 ,:]

    skill_list =[skill_z, skill_t, skill_t2m, skill_q, skill_ws]
    rmse_list = [rmse_z, rmse_t, rmse_t2m, rmse_q, rmse_ws]
    # cmap,clevs = meb.def_cmap_clevs(meb.cmaps.me,clevs=[-50,-25,-15,-10,-5,-2,-1,1,2,5,10,15,25,50])
    fig = plt.figure(figsize=(12.5, 2.5), dpi=600)
    plt.subplots_adjust(wspace=0.05)
    annot_list = [0, 1, 1, 1, 1]
    for k in range(5):
        ax_one = plt.subplot(1, 5, k + 1)
        data_k = copy.deepcopy(skill_list[k])
        nx = data_k.shape[1]
        ny = data_k.shape[0]
        x = np.arange(nx + 1) - 0.5
        y = np.arange(ny + 1) - 0.5
        norm = BoundaryNorm(clevs, ncolors=cmap.N + 1, extend="both")
        im = ax_one.pcolormesh(x, y, data_k, cmap=cmap, norm=norm, linewidths=2, edgecolor="w")
        ax_one.spines['bottom'].set_linewidth(0)
        ax_one.spines['top'].set_linewidth(0)
        ax_one.spines['right'].set_linewidth(0)
        ax_one.spines['left'].set_linewidth(0)
        fmt_tag = "%." + str(annot_list[k]) + "f"
        data = copy.deepcopy(rmse_list[k])

        for j in range(ny):
            for i in range(nx):
                data_ijk = data_k[j, i]
                if not np.isnan(data_ijk):
                    ax_one.text(i, j, fmt_tag % data[j, i], ha="center", va="center", fontsize=10, c="k")
                rect = patches.Rectangle((i - 0.45, j - 0.45), 0.9, 0.9, linewidth=0.1, edgecolor='gray',
                                         facecolor='none')
                ax_one.add_patch(rect)
            rect = patches.Rectangle((-0.45, j - 0.45), 4.9, 0.9, linewidth=0.3, edgecolor='gray', facecolor='none')
            ax_one.add_patch(rect)
        # ax_one.set_xticks(np.arange(nx))
        plt.xticks(np.arange(nx), xlabels, fontsize=11, family='Times New Roman')
        plt.xlabel("Lead time[days]", fontsize=12)
        if k == 0:
            plt.yticks(np.arange(5).tolist(), ylabels, fontsize=12)
        else:
            plt.yticks([])
        plt.title(titles_little[k], fontsize=8)
        plt.text(2, 5.3, titles[k], fontsize=14, ha="center", va="center")

    location = [0.2, -0.15, 0.6, 0.05]
    colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
    plt.colorbar(im, cax=colorbar_position, orientation="horizontal",
                 ticks=clevs, label="Better <- % difference in RMSE vs ECMWF -> Worse")

    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches="tight")
    if show:
        plt.show()
    plt.close()

if __name__ =="__main__":
    import pandas as pd
    import meteva.method as mem
    import meteva.perspact as mps  # 透视分析模块
    middle_result_path = r"H:\task\other\202308-AImodel\mid\z_tase_cli.h5"
    df_tase_z_cli = pd.read_hdf(middle_result_path)
    rmse_z500, gdict = mps.score_df(df_tase_z_cli, mem.rmse,
                                    s={"time_range": ["2022040108", "2023040108"], "level": 500,
                                       "dtime": [24, 72, 120, 168, 240]},
                                    g=["member", "dtime"],
                                    gll_dict={"member": ["ECMWF", "PANGU", "FENGWU", "FUXI", "Climatology"]})

    middle_result_path = r"H:\task\other\202308-AImodel\mid\t_tase_cli.h5"
    df_tase_t_cli = pd.read_hdf(middle_result_path)
    rmse_t850, gdict = mps.score_df(df_tase_t_cli, mem.rmse,
                                    s={"time_range": ["2022040108", "2023040108"], "level": 850,
                                       "dtime": [24, 72, 120, 168, 240]},
                                    g=["member", "dtime"],
                                    gll_dict={"member": ["ECMWF", "PANGU", "FENGWU", "FUXI", "Climatology"]})

    middle_result_path = r"H:\task\other\202308-AImodel\mid\speed_tase_cli.h5"
    df_tase_speed_cli = pd.read_hdf(middle_result_path)
    rmse_speed850, gdict = mps.score_df(df_tase_speed_cli, mem.rmse,
                                        s={"time_range": ["2022040108", "2023040108"], "level": 850
                                            , "dtime": [24, 72, 120, 168, 240]}, g=["member", "dtime"]
                                        , gll_dict={"member": ["ECMWF", "PANGU", "FENGWU", "FUXI", "Climatology"]})

    middle_result_path = r"H:\task\other\202308-AImodel\mid\q_tase_cli.h5"
    df_tase_q_cli = pd.read_hdf(middle_result_path)
    rmse_q700, gdict = mps.score_df(df_tase_q_cli, mem.rmse, s={"time_range": ["2022040108", "2023040108"], "level": 850
        , "dtime": [24, 72, 120, 168, 240]}, g=["member", "dtime"]
                                    , gll_dict={"member": ["ECMWF", "PANGU", "FENGWU", "FUXI", "Climatology"]})

    middle_result_path = r"H:\task\other\202308-AImodel\mid\t2m_tase_cli.h5"
    df_tase_t2m_cli = pd.read_hdf(middle_result_path)
    rmse_t2m, gdict = mps.score_df(df_tase_t2m_cli, mem.rmse, s={"time_range": ["2022040108", "2023040108"], "level": 2,
                                                                 "dtime": [24, 72, 120, 168, 240]},
                                   g=["member", "dtime"],
                                   gll_dict={"member": ["ECMWF", "PANGU", "FENGWU", "FUXI", "Climatology"]})

    rmse_skill_seaborn(rmse_z500, rmse_t850, rmse_q700, rmse_speed850, rmse_t2m, gdict["member"],gdict["dtime"],
                  save_path=r"H:\task\other\202308-AImodel\png\comprehensive.png")


import numpy as np
from meteva.method.space.fqi.lib.locmeasures2d_prep import locmeasures2d_prep
from meteva.method.space.fqi.lib.util import get_attributes
from meteva.method.space.fqi.lib.datagrabber_spatialVx import datagrabber_spatialVx
from meteva.method.space.fqi.lib.im import im
from meteva.method.space.fqi.lib.solutionset import solutionset
from meteva.method.space.fqi.lib.cbind import cbind
from meteva.method.space.fqi.uiqi import uiqi
from meteva.method.space.fqi.lib.locperf import locperf
from meteva.method.space.fqi.lib.surrogater2d import surrogater2d
from meteva.method.space.fqi.lib.locperfer import locperfer
#import pyreadr
from meteva.method.space.baddeley_binary_image_metric.lib.make_spatialVx import make_spatialVx
#import rdata
import meteva

def fqi(grd_ob,grd_fo, thresholds):
    obs_array = grd_ob.values.squeeze()
    fst_array = grd_fo.values.squeeze()
    z = surrogater2d(obs_array, zero_down=True, n=10)
    hold = make_spatialVx(obs_array, fst_array,loc=None, thresholds=thresholds)
    result = fqi_origin(hold, surr=z, k=4)
    fqi_array = result["fqi"]
    fqi_array[np.isnan(fqi_array)] = meteva.base.IV
    return result["fqi"]


def fqi_origin(input_object, surr=None, k=4, time_point=0, obs=0, model=0):
    out = {}
    input_object = locmeasures2d_prep(input_object=input_object, k=k)
    attributes = get_attributes(input_object)
    out["attributes"] = attributes
    dat = datagrabber_spatialVx(input_object, time_point=time_point, obs=obs, model=model)
    x = np.array(dat["X"])
    y = np.array(dat["Xhat"])
    out["attributes"]["data_name"] = [attributes["dataname"], attributes["obsname"][obs],
                                      attributes["modelname"][model]]
    thresholds = attributes["thresholds"]
    thresholds = cbind(thresholds[0][:, obs], thresholds[1][:, model])
    out["attributes"]["thresholds"] = thresholds
    if surr is None:
        surr = surrogater2d(im=x)
    xdim = x.shape
    xim = im(x)
    yim = im(y)
    q = np.array(thresholds).shape[0]
    ks = attributes["k"]
    nk = 1
    phd_norm = np.full((nk, q), np.nan)
    fqi = np.full((nk, q), np.nan)
    uiqi_norm = np.full((q, 1), np.nan)
    for threshold in range(q):
        temp_x = xim['v'] >= thresholds[threshold, 0]
        xim['v'] = temp_x
        ix = solutionset(xim)
        temp_y = yim['v'] >= thresholds[threshold, 1]
        yim['v'] = temp_y
        iy = solutionset(yim)
        idx = x >= thresholds[threshold, 0]
        idy = y >= thresholds[threshold, 1]
        x2 = x
        x2[~idx] = 0
        y2 = y
        y2[~idy] = 0
        denom = uiqi(x=x2, xhat=y2, only_nonzero=True)["uiqi"]
        uiqi_norm[threshold] = denom
        for k in range(nk):
            num1 = locperf(X=ix, Y=iy, which_stats="ph", k=ks)["ph"]
            tmp = np.full(surr.shape[2], np.nan, dtype=float)
            for l in range(surr.shape[2]):
                if isinstance(ks, int):
                    tmpK = ks
                else:
                    tmpK = ks[k]
                tmp[l] = locperfer(surr[:, :, 0], ix, thresholds[threshold, 0], tmpK)
            num2 = np.nanmean(tmp)
            if num1 is np.nan or num2 is np.nan or num1 is None or num2 is None:
                num_tmp = np.nan
            else:
                num_tmp = num1 / num2
            phd_norm[k, threshold] = num_tmp
            fqi[k, threshold] = num_tmp / denom
    out["phd_norm"] = phd_norm
    out["uiqi_norm"] = uiqi_norm
    out["fqi"] = fqi
    out["class"] = "fqi"
    return out


if __name__ == '__main__':
    # parsed = rdata.parser.parse_file('/home/andyji/hold.rdata')
    # hold = rdata.conversion.convert(parsed)['hold']
    #
    # h1 = np.fft.ifft(hold)
    # h2 = np.fft.ifft(hold.flatten()).reshape(hold.shape)
    # # temp2 = fft2d(temp, bigdim=(1024, 512), inverse=True) / 301101
    # geom000 = pyreadr.read_r('../fuzzy_logic/data/geom000.Rdata')['geom000']
    # geom001 = pyreadr.read_r('../fuzzy_logic/data/geom001.Rdata')['geom001']
    # ICPg240Locs = pyreadr.read_r('../fuzzy_logic/data/ICPg240Locs.Rdata')['ICPg240Locs']
    # ix = np.array([[0,0,1],[1,0,1],[1,1,1]])
    # iy = np.array([[0, 0, 1], [1, 0, 1], [1, 1, 1]])

    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    path_fo_27 = r'H:\test_data\input\mem\mode\ec\rain03\20072508.027.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo_03 = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    grd_fo_27 = meb.read_griddata_from_nc(path_fo_27, grid=grid1, time="2020072508", dtime=27, data_name="ECMWF")
    result = fqi(grd_fo_03,grd_fo_27,thresholds=[0.01,50.01])
    #
    # obs_array = grd_ob.values.squeeze()
    # fst_array = grd_fo_03.values.squeeze()
    # z = surrogater2d(obs_array, zero_down=True, n=10)
    # hold = make_spatialVx(obs_array, fst_array,loc=None, thresholds=[0.01, 50.01])
    # result = fqi(hold, surr=z, k=4)
    print(result)



from . import score
from . import table
from . import plot
from . import skill

from .table import contingency_table_yesorno
from .plot import performance,performance_mr_far
from .skill import pc_skill,bias_extend_skill,ts_skill,spo,sbi,sfa,spc,sts
from .score import hfmc,hfmc_of_sun_rain,pofd_hfmc,pod_hfmc,sr_hfmc,bias_hfmc,ts_hfmc,ets_hfmc,far_hfmc,mr_hfmc,pc_hfmc,pc_of_sun_rain_hfmc,r_hfmc,s_hfmc,dts,dts_hfmc
from .score import pod,pofd,sr,bias,ts,ets,far,mr,pc,pc_of_sun_rain,r,s,bias_extend_linear,bias_extend_log,hk_yesorno_hfmc,hk_yesorno,hss_yesorno,hss_yesorno_hfmc
from .score import hap_count,effective_dtime,effective_dtime_hfmdt,hfmdt
from .score import ob_fo_hr,ob_fo_hr_hfmc,ob_fo_hc_hfmc,ob_fo_hc,odds_ratio,odds_ratio_hfmc,orss_hfmc,orss,fscore,fscore_hfmc
from .score import  false_alarm_count,false_alarm_count_hfmc


import copy
import os
import numpy as np
import traceback
import pandas as pd
import datetime
import meteva
import json
import gzip

def write_stadata_to_micaps3(sta0,save_path = "a.txt",creat_dir = False, type = -1,effectiveNum = 4,show = False,title = None):
    """
    生成micaps3格式的文件
    :param sta0:站点数据信息
    :param save_path 需要保存的文件路径和名称
    :param type 类型：默认：1
    :param effectiveNum 有效数字 默认为：4
    :return:保存为micaps3格式的文件
    """
    try:
        sta = copy.deepcopy(sta0)
        dir = os.path.split(os.path.abspath(save_path))[0]
        if not os.path.isdir(dir):
            if not creat_dir:
                print("文件夹：" + dir + "不存在")
                return False
            else:
                meteva.base.tool.path_tools.creat_path(save_path)

        br = open(save_path,'w')
        end = len(save_path)
        start = max(0, end-16)
        nsta =len(sta.index)
        time = sta['time'].iloc[0]
        if isinstance(time,np.datetime64) or isinstance(time,datetime.datetime):
            time_str = meteva.base.tool.time_tools.time_to_str(time)
            time_str = time_str[0:4] + " " +time_str[4:6] + " " + time_str[6:8] + " " + time_str[8:10] + " "
        else:
            time_str = "2099 01 01 0 "

        if np.isnan(sta['level'].iloc[0]):
            level = 0
        else:
            level = int(sta['level'].iloc[0])
        if type<0 or level == np.NaN or level ==pd.NaT:
            level = int(type)

        if title is None:
            str1=("diamond 3 " + save_path[start:end] + "\n"+ time_str + str(level) +" 0 0 0 0\n1 " + str(nsta) + "\n")
        else:
            str1 = ("diamond 3 " + title + "\n" + time_str + str(level) + " 0 0 0 0\n1 " + str(
                nsta) + "\n")
        br.write(str1)
        br.close()
        data_names = meteva.base.basicdata.get_stadata_names(sta)
        if "alt" not in data_names:
            data_name = meteva.base.basicdata.get_stadata_names(sta)[0]
            df = copy.deepcopy(sta[['id','lon','lat',data_name]])
            df['alt'] = 0
            df = df.reindex(columns=['id', 'lon', 'lat', 'alt', data_name])
        else:
            colums = ['id','lon','lat','alt']
            for name in data_names:
                if name != "alt":
                    colums.append(name)
                    break
            df = sta.loc[:,colums]
            if len(colums) == 4:
                df["data0"] = 0
        effectiveNum_str = "%." + '%d'% effectiveNum + "f"
        # ---by zhyn
        if title is None:
            df.to_csv(save_path,mode='a',header=None,sep = "\t",float_format=effectiveNum_str,index = None)
        else:
            if title.find("闪电")>=0 or title.find("light")>=0:
                df.to_csv(save_path, mode='a', header=None, sep="  ", float_format=effectiveNum_str, index=None)
            else:
                df.to_csv(save_path, mode='a', header=None, sep="\t", float_format=effectiveNum_str, index=None)
        # ---end by zhyn
        if show:
            print('成功输出至' + save_path)
        return True
    except:
        exstr = traceback.format_exc()
        print(exstr)
        return False

def tran_stadata_to_gds_flow(sta):
    discriminator = b"mdfs"
    data_type =3
    data_type_byte = np.ndarray.tobytes(np.array([data_type]).astype(np.int16))

    description =""
    if "description" in sta.attrs.keys():
        description =  description + sta.attrs["description"]
    description = description.encode(encoding='utf-8')
    if len(description) < 100:
        description = description + np.ndarray.tobytes(np.zeros(100 - len(description)).astype(np.int8))


    level = np.ndarray.tobytes(np.array([sta.iloc[0,0]]).astype(np.float32))

    levelDescription = ""
    if "levelDescription" in sta.attrs.keys():
        levelDescription = sta.attrs["levelDescription"]
    levelDescription = levelDescription.encode(encoding='utf-8')
    if len(levelDescription) < 50:
        levelDescription = levelDescription + np.ndarray.tobytes(np.zeros(50 - len(levelDescription)).astype(np.int8))

    time0 = meteva.base.all_type_time_to_datetime(sta["time"].values[0])
    y = time0.year
    m = time0.month
    d = time0.day
    h = time0.hour
    min = time0.minute
    se = time0.second

    y_m_d_h_M_s_timezone = np.ndarray.tobytes(np.array([y,m,d,h,min,se,8]).astype(np.int32))
    blank = np.ndarray.tobytes(np.zeros(100).astype(np.int8))

    nsta = len(sta.index)
    station_number = np.ndarray.tobytes(np.array([nsta]).astype(np.int32))
    data_names = meteva.base.get_stadata_names(sta)
    nele = len(data_names)
    value_bytes = np.ndarray.tobytes(np.array([nele]).astype(np.int16))

    for j in range(nele):
        value_bytes += np.ndarray.tobytes(np.array([601,5]).astype(np.int16))

    for i in range(nsta):
        value_bytes += np.ndarray.tobytes(np.array([sta.iloc[i,3],sta.iloc[i,4],sta.iloc[i,5]]).astype(np.float32))
        value_bytes += np.ndarray.tobytes(np.array(nele).astype(np.int16))
        for j in range(nele):
            value_bytes += np.ndarray.tobytes(np.array([601]).astype(np.int16))
            value_bytes += np.ndarray.tobytes(np.array([sta.iloc[i,6+j]]).astype(np.float32))


    bytes1 = discriminator+data_type_byte+description+level+levelDescription+y_m_d_h_M_s_timezone
    bytes2 = blank+station_number+value_bytes
    bytes = bytes1+bytes2

    return bytes

def write_stadata_to_gds_file(da,save_path = "a.txt",creat_dir = False,show = False):
    try:
        dir = os.path.split(os.path.abspath(save_path))[0]
        if not os.path.isdir(dir):
            if not creat_dir:
                print("文件夹："+dir+"不存在")
                return False
            else:
                meteva.base.tool.path_tools.creat_path(save_path)

        bytes = tran_stadata_to_gds_flow(da)
        br = open(save_path, 'wb')
        br.write(bytes)
        br.close()
        if show:
            print('成功输出至' + save_path)
        return True
    except:
        exstr = traceback.format_exc()
        print(exstr)
        return False

def pretty_floats(obj,effective_num):
    if isinstance(obj, float):
        return round(obj, effective_num)
    elif isinstance(obj, dict):
        return dict((k, pretty_floats(v,effective_num)) for k, v in obj.items())
    elif isinstance(obj, (list, tuple)):
        return map(pretty_floats, obj)
    return obj

def stadata_to_json(sta,effective_num):
    data_names = meteva.base.get_stadata_names(sta)
    sta1 = sta.round(effective_num)
    #print(effective_num)
    #print(sta1)
    meteva.base.set_stadata_names(sta1,["data0"])
    dict_attrs = copy.deepcopy(sta.attrs)
    dict_attrs["time"] = meteva.base.all_type_time_to_str(sta1['time'].values[0])
    dict_attrs["dtime"] = str(sta1['dtime'].values[0])
    dict_attrs["level"] = str(sta1['level'].values[0])
    dict_attrs["model"] = data_names[0]
    sta1 = sta1.drop(["level","time","dtime"],axis = 1)
    if  "data_start_columns" not in dict_attrs.keys():
        dict_attrs["data_start_columns"] = 6
    dict1 = {}
    dict1["attrs"] =dict_attrs
    #print(dict_attrs)
    dict1["data"] = sta1.to_dict(orient='index')
    json_str = json.dumps(pretty_floats(dict1,effective_num))
    return json_str


def stadata_to_json_bytes(sta,effective_num):
    json_str = stadata_to_json(sta,effective_num)
    json_bytes = gzip.compress(json_str.encode("GBK"))
    return json_bytes

def write_stadata_to_json_file(sta,filename,effective_num = 3):
    br = open(filename, 'w')
    json_str = stadata_to_json(sta,effective_num)
    br.write(json_str)
    br.close()

def write_stadata_to_json_byte_file(sta,filename,effective_num = 3):
    br = open(filename, 'wb')
    json_bytes = stadata_to_json_bytes(sta,effective_num)
    br.write(json_bytes)
    br.close()

def put_stadata_to_micaps(sta,effective_num = 3,layer_description = None):

    import MPython
    pyclient = MPython.MICAPSPython()
    if (pyclient.GetConnectStatus()):
        print("MICAPS4客户端已连接")
    else:
        pyclient.ReConnect()

    data_names = meteva.base.get_stadata_names(sta)
    nrow = len(data_names)

    if nrow ==1:
        if layer_description is not None:
            if isinstance(layer_description,list):
                sta.attrs["data_source"] = layer_description[0]
                #sta.attrs["layer_description"] = layer_description[0]
            else:
                #sta.attrs["layer_description"] = layer_description
                sta.attrs["data_source"] = layer_description
        else:
            #sta.attrs["layer_description"] = ""
            pass
        if "model" not in sta.attrs.keys():
            sta.attrs["model"] = data_names[0]
        json_bytes = stadata_to_json_bytes(sta, effective_num)
        pyclient.CreateDiscreteLayer(json_bytes)
    else:
        for i in range(nrow):
            sta1 = meteva.base.in_member_list(sta,[data_names[i]])
            if layer_description is not None:
                sta1.attrs["data_source"] = layer_description[i]
                #sta1.attrs["layer_description"] = layer_description[i]
            else:
                #sta1.attrs["layer_description"] = ""
                pass
            if "model" not in sta1.attrs.keys():
                sta1.attrs["model"] = data_names[i]

            json_bytes = stadata_to_json_bytes(sta1,effective_num)
            pyclient.CreateDiscreteLayer(json_bytes)

'''def sta_to_json(sta,effective_num):
    sta1 = sta.round(effective_num)
    sta1.attrs = copy.deepcopy(sta.attrs)
    sta1["time"]=sta1['time'].apply(lambda x: x.strftime('%Y%m%d%H%M'))
    dict1 = {}
    dict1["attrs"] = sta1.attrs
    dict1["data"] = sta1.to_dict(orient='index')
    str_data = json.dumps(dict1)
    return str_data'''


def write_stadata_to_micaps2(sta_speed,sta_angle,save_path = "a.txt",creat_dir = False, type = 0,effectiveNum = 4,show = False,title = None):
    """
    生成micaps3格式的文件
    :param sta0:站点数据信息
    :param save_path 需要保存的文件路径和名称
    :param type 类型：默认：1
    :param effectiveNum 有效数字 默认为：4
    :return:保存为micaps3格式的文件
    """
    try:

        dir = os.path.split(os.path.abspath(save_path))[0]
        if not os.path.isdir(dir):
            if not creat_dir:
                print("文件夹：" + dir + "不存在")
                return False
            else:
                meteva.base.tool.path_tools.creat_path(save_path)

        br = open(save_path,'w')
        end = len(save_path)
        start = max(0, end-16)
        sta = meteva.base.combine_on_all_coords(sta_angle,sta_speed)
        nsta =len(sta.index)
        time = sta['time'].iloc[0]
        if isinstance(time,np.datetime64) or isinstance(time,datetime.datetime):
            time_str = meteva.base.tool.time_tools.time_to_str(time)
            time_str = time_str[0:4] + " " +time_str[4:6] + " " + time_str[6:8] + " " + time_str[8:10] + " "
        else:
            time_str = "2099 01 01 0 "

        if np.isnan(sta['level'].iloc[0]):
            level = 0
        else:
            level = int(sta['level'].iloc[0])
        if type<0 or level == np.NaN or level ==pd.NaT:
            level = int(type)

        if title is None:
            str1=("diamond 2 " + save_path[start:end] + "\n"+ time_str + str(level) +" " + str(nsta) + "\n")
        else:
            str1 = ("diamond 2 " + title + "\n" + time_str + str(level) +" "  + str(nsta) + "\n")
        br.write(str1)
        br.close()

        df = copy.deepcopy(sta[['id','lon','lat']])
        df['alt'] = 0
        df["d0"] = 1
        df["d1"] = 9999
        df["d2"] = 9999
        df["d3"] = 9999
        df["angle"] = sta.iloc[:,6]
        df["speed"] = sta.iloc[:, 7]

        effectiveNum_str = "%." + '%d'% effectiveNum + "f"
        df.to_csv(save_path,mode='a',header=None,sep = "\t",float_format=effectiveNum_str,index = None)
        if show:
            print('成功输出至' + save_path)
        return True
    except:
        exstr = traceback.format_exc()
        print(exstr)
        return False



if __name__ == "__main__":
    path = r"D:\book\test_data\charpter11\ob_with_noisy\22010102.000"
    sta = pd.read_csv(path, parse_dates=["time"])
    speed,angle = meteva.base.wind_to_speed_angle(sta)
    write_stadata_to_micaps2(speed,angle,r"D:\book\test_data\charpter11\a.txt")


from meteva.method.space.fuzzy_logic.lib.hoods2d import hoods2d
#import pyreadr
from meteva.method.space.fuzzy_logic.lib.make_spatialVx import make_spatialVx
import numpy as np

def joint(grd_ob,grd_fo,half_window_sizes, thresholds = None,compare=">=", show=False):

    obs_array = grd_ob.values.squeeze()
    fst_array = grd_fo.values.squeeze()
    hold = make_spatialVx(obs_array,fst_array,thresholds=thresholds)
    levels = []
    for s in half_window_sizes:
        levels.append(s*2+1)

    result = joint_origin(hold,levels=levels,rule=compare,verbose=show)
    return result


def joint_origin(input_object, levels, time_point=1,
          obs=0, model=0, max_n=None, rule=">=", verbose=False):
    result = hoods2d(input_object, which_methods=["joint"], time_point=time_point,
                     obs=obs, model=model, levels=levels, max_n=max_n, rule=rule, verbose=verbose)["joint"]
    result["thresholds_ob"] = input_object["thresholds"][0]
    result["thresholds_fo"] = input_object["thresholds"][1]
    result["window_sizes"] = levels
    return result


if __name__ == '__main__':
    import meteva.base as meb
    import meteva.method as mem
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo_03 = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    look = mem.fuzzy_logic.joint(grd_ob, grd_fo_03,thresholds=[1], half_window_sizes=[8])
    #look = mem.fuzzy_logic.joint(grd_ob, grd_fo_03, half_window_sizes=[1, 2, 4, 8])
    print(look)


from . import datagrabber_spatialVx
from . import fss2dfun
from . import fuzzyjoint2dfun
from . import hoods2d
from . import hoods2dPrep
from . import hoods2dSetUpLists
from . import hoods2dsmooth
from . import kernel2dmeitsjer
from . import kernel2dsmooth
from . import make_spatialVx
from . import mincvg2dfun
from . import multicon2dfun
from . import progmatic2dfun
from . import thresholder
from . import  thresholder_spatialVx
from . import vxstats
from . import zapsmall


import numpy as np
from meteva.method.space.fqi.lib.fft2d import fft2d
from meteva.method.space.fqi.lib.aaft2d import aaft2d
from meteva.method.space.fqi.lib.zapsmall import zapsmall
import scipy.stats as ss
from .mae import mae
#import pyreadr


def surrogater2d(im, frac=0.95, n=10, maxiter=100, zero_down=True, verbose=False):
    im = np.array(im)
    xdim = np.shape(im)
    new_dim = xdim + (n, )
    out = np.full(new_dim, np.nan)
    im_vec = im
    tmp = fft2d(im)
    bigdim = tmp["bigdim"]
    sm_n = np.prod(xdim)
    big_n = np.prod(bigdim)
    amps = np.absolute(tmp["fft"])
    for i in range(n):
        sur = aaft2d(im=im, bigdim=bigdim)
        err_min = 0
        past_err = 0
        err = 0
        surmin = np.full(xdim, np.nan)
        if verbose:
            print(
                "\nFound initial surrogate via AAFT method.  Iterating to try to get same power spectrum and distribution as im.\n")
        for k in range(maxiter):
            if verbose:
                print(k)
            temp = fft2d(sur, bigdim=bigdim)
            ampsnew = np.absolute(temp)
            ind = ampsnew == 0
            if np.sum(ind) > 0:
                ampsnew[ind] = 1e-12
            ampsadj = frac * amps + (1 - frac) * ampsnew
            temp = temp * ampsadj / ampsnew
            temp2 = fft2d(temp, bigdim=bigdim, inverse=True) / sm_n
            sur = np.real(temp2)
            if zero_down:
                sur[sur < 0] = 0
                sur = zapsmall(sur)
            if k == 0:
                past_err = np.inf
                err_min = np.inf
            err = mae(amps, ampsnew)
            if err < err_min:
                if verbose:
                    print("\nerr = " + err + " < err_min = " + err_min + "\n")
                surmin = sur
                err_min = err
            if np.isinf(past_err):
                past_err = err
                continue
            elif np.abs((err - past_err) / past_err) * 100 <= 1:
                break
            past_err = err
        if verbose:
            print("\nFinished iterating.  Final Err = " + err + "\n")
        out[:, :, i] = surmin
        if verbose:
            print("\nFound surrogate " + i + "\n")
    return out



if __name__ == '__main__':
    #geom000 = pyreadr.read_r('../data/geom000.Rdata')['geom000']
    #z = surrogater2d(geom000, zero_down = True, n = 10)
    #print("h")
    pass


from meteva.method.space.geometric_characterizations.lib.util import get_attributes
from meteva.method.space.geometric_characterizations.lib.datagrabber_spatialVx import datagrabber_spatialVx
import pandas as pd
from meteva.method.space.geometric_characterizations.aindex import aindex
from meteva.method.space.geometric_characterizations.cindex import cindex
from meteva.method.space.geometric_characterizations.sindex import sindex


def spatial_index(input_object, func_type, thresh=None, dx=1, dy=1, time_point=1, obs=1, model=1):
    attributes = get_attributes(input_object)
    dat = datagrabber_spatialVx(input_object, time_point=time_point - 1, obs=obs - 1, model=model - 1)
    if isinstance(input_object["time"], list):
        attributes["time"] = input_object["time"][time_point]
    else:
        attributes["time"] = input_object["time"]
    if isinstance(input_object["obsname"], list):
        attributes["obs_name"] = input_object["obsname"][obs]
    else:
        attributes["obs_name"] = input_object["obsname"]
    if isinstance(input_object["modelname"], list):
        attributes["model_name"] = input_object["modelname"][model]
    else:
        attributes["model_name"] = input_object["modelname"]
    grd_ob = dat["X"]
    grd_fo = dat["Xhat"]
    if func_type == "a":
        res1 = aindex(grd_ob, thresh=thresh, dx=dx, dy=dy)
        res2 = aindex(grd_fo, thresh=thresh, dx=dx, dy=dy)
    elif func_type == "c":
        res1 = cindex(grd_ob, thresh=thresh)
        res2 = cindex(grd_fo, thresh=thresh)
    elif func_type == "s":
        res1 = sindex(grd_ob, thresh=thresh, loc=attributes["loc"])
        res2 = sindex(grd_fo, thresh=thresh, loc=attributes["loc"])
    else:
        raise Exception("func_type错误")
    res = pd.DataFrame({attributes["obs_name"]: res1, attributes["model_name"]: res2}, index=[0])
    return res
